/*
 *	Copyright (C) 2007, 2008, 2009. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvdefaultgui.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVDEFAULTGUI_H
#define QVDEFAULTGUI_H

#include <QVGUI>

#include <QVBoxLayout>
#include <QMenuBar>
#include <QMenu>
#include <QGroupBox>
#include <QToolBox>

/*!
@class QVDefaultGUI qvdefaultgui.h qvdefaultgui/qvdefaultgui.h
@brief Main user interface widget for QVision applications.

This is the default graphical user interface class. It offers basic widgets to control several aspects of the execution of a QVision application. More specifically, this default GUI will show two separated tabbed toolbox widgets: one for workers, with a control button bar and a parameter inspector for each QVWorker object in the application, and another one for cameras, which also provides control and input/output information for each QVCameraWorker. The main widget has also a menu with typical Quit, Window and Help submenus. Particularly, the window submenu allows show/hide control of every window (@ref QVImageCanvas, @ref QVGLCanvas,
and @ref QVPlot ) created by the QVision application.

It is very easy to use this class: you simply must declare an instance of it in the main function of your application (after the QVApplication object declaration). Then, when the application enters its exec() method, this instance of QVDefautlGUI will detect all the involved workers, cameras, canvas and so on, and will automatically create the needed visual interface:

\code
int main(int argc, char *argv[])
        {
        QVApplication app(argc, argv, "An example application");
        QVDefaultGUI interface;
	// Declaration and linking of workers, cameras, canvas, etc. of our application would be here...
        return app.exec();
        }
\endcode

This class is one of the implementations of the pure virtual base class @ref QVGUI provided by the QVision framework. Maybe you are also interested in the @ref QVDesignerGUI one.

@image html qvdefaultgui.png

@ingroup qvgui
*/
class QVDefaultGUI: public QWidget, public QVGUI
{
	Q_OBJECT
public:

	/// @brief Constructs a QVDefaultGUI object.
	/// @param parent (Optional) parent for the created widget
	QVDefaultGUI(QWidget *parent = 0);

#ifndef DOXYGEN_IGNORE_THIS

	/// @brief Inits the QVDefaultGUI object, adding all workers, cameras and windows in the QVApplication to the interface.
	void init();

	void show() { QWidget::show(); }

signals:
	/// @brief Emitted when the image canvas window closes.
	void closed();

public slots:
	/// @brief Shows a modal text window with the application help.
	void showHelp();

protected:
	void closeEvent(QCloseEvent *event);

private:
	QVBoxLayout *vboxlayout;
	QMenuBar *menubar;
	QMenu *windowmenu;
	QGroupBox *groupboxcameras,*groupboxworkers;
	QToolBox *toolboxcameras,*toolboxworkers;

#endif
};

#endif
