/*
 *	Copyright (C) 2007, 2008, 2009. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvcameraworker.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVCAMERAWORKER_H
#define QVCAMERAWORKER_H

#include <QVWorker>
#include <QVImage>

/*!
@class QVCameraWorker qvcameraworker.h qvio/qvcameraworker.h
@brief Virtual base class to create input video reader worker components for the QVision application architecture.

This class is pure virtual, and should not be used by normal QVision users (except when implementing their own video sources workers, such as, say, direct driver-based IEEE 1394 or V4L2 cameras; remember, though, that @ref QVMPlayerCameraWorker can read almost any kind of video input source, including live cameras, such as webcams, DV cameras, and so on). Any of its two derived classes @ref QVYUV4MPEG2CameraWorker and @ref QVMPlayerCameraWorker are the classes that normal users will use. See documentation on those classes for more details.

Implementors of new kind of QVCameraWorkers should only have to redefine this five pure virtual methods:

- virtual bool openCam(QString urlName, int &cols, int &rows, int &fps) = 0;
- virtual void closeCam() = 0;
- virtual bool grab(QVImage<uChar,1> &imgY, QVImage<uChar,1> &imgU, QVImage<uChar,1> &imgV) = 0;
- virtual double lengthOfVideo() = 0;
- virtual double currentPos() = 0;

Whose desired behaviour should be self-explanatory: the first one should open the video source specified by the string urlName, trying to open it with the suggested cols, rows and fps. The method must indicates a successful opening by returning true (and updating the corresponding cols, rows and fps output values to the really obtained ones, which could coincide or not with the suggested input values), or false, if there was any problem opening the source. The second must close the video source. The third must grab Y, U and V channel images in the corresponding variable parameters, with respective sizes cols x rows, cols/2 x rows/2 and cols/2 x rows/2 (with cols and rows as returned by the openCam method), returning true if grabbing was succesful, or false otherwise (for example, if the camera was closed). The fourth method must return the length of the video in seconds (or 0.0 if not available; for example, for live video cameras). Finally, the fifth method must return, when called, the current position in the video, again in seconds.

All camera workers inheriting from this class will always have the following properties:

Input properties:

- <i>NoLoop</i> (bool): If the camera should be opened in no loop mode.
- <i>RealTime</i> (bool): If the camera should be opened in real time mode.
- <i>URL</i> (QString): URL of the video file to read.
- <i>Cols</i> (int): Number of suggested columns for the video source.
- <i>Rows</i> (int): Number of suggested rows for the video source.

<b>Note</b>.- RealTime property is also input (and as such can be initialized using the command line), but invisible to the GUI (because once the camera has been linked, it cannot be relinked changing its synchronism).

<b>Note</b>.- Note that you can open the camera in a different size of that specified in the real video source file, by setting the desired values of properties <i>Cols</i> and <i>Rows</i> (this class will automatically rescale output images if needed).

Output properties are the following:

- <i>Opened</i> (bool): Tells if the camera is correctly opened and working.
- <i>FPS</i> (int): FPS of the video.
- <i>Frames</i> (int): Number of read frames up to now.
- <i>ColsR</i> (int): Actual number of columns of the video -not necessarily equal to the number of requested columns, which will in any case be transparently rounded to the nearest upper even integer-.
- <i>RowsR</i> (int): Actual number of rows of the video -not necessarily equal to the number of requested rows, which will in any case be transparently rounded to the nearest upper even integer-.
- <i>Pos</i> (double): Current position in the video, if available (in seconds).
- <i>Length</i> (double): Length of the video, if available (in seconds).


<b>Note</b>.- Remember that you can open the camera in a different size of that specified in the real video source file (this class will automatically rescale output images if needed); thus, <i>ColsR</i> and <i>RowsR</i> will always contain the real size of the output images served by this worker.

Output image properties:

- <i>RGB image</i> (QVImage<uChar,3>): Last grabbed RGB image.
- <i>Y channel image</i> (QVImage<uChar,1>): Last grabbed Y channel image.
- <i>U channel image</i> (QVImage<uChar,1>): Last grabbed U channel image.
- <i>V channel image</i> (QVImage<uChar,1>): Last grabbed V channel image.

Output image properties can (and should!) be linked automatically, without the need of naming them explicitly, using any of the @ref QVCameraWorker::linkProperty() available methods.

@ingroup qvio
*/

class QVCameraWorker: public QVWorker
	{
	Q_OBJECT
	public:
		/// @brief Constructor for (virtual) QVCameraWorker class.
		///
		/// @param name the (optional) name given to the camera worker.
		QVCameraWorker(QString name = QString(""));

		/// @brief Destructor for (virtual) QVCameraWorker class.
		~QVCameraWorker();

		/// @brief Links a QVImage<uChar,3> or QVImage<uChar,1> from the camera to another worker.
		///
		/// This method will automatically link, using the adequate synchronism (Asynchronous for real time cameras
		/// and Synchronous for non real time cameras), a gray or an RGB image in the worker to the corresponding output
		/// image property of the camera. An error (return value=false) will occur if dstProp is not of the adequate
		/// type (QVImage<uChar,3> or QVImage<uChar,1>).
		///
		/// @param dstCont Pointer to the destination worker.
		/// @param dstProp Name of the destination property.
		/// @return true if success, false otherwise.
		bool linkProperty(QVPropertyContainer *dstCont, QString dstProp);

		/// @brief Links 3 QVImage<uChar,1> from the camera (YUV channels) to the corresponding ones of another worker.
		///
		/// This method will automatically link, using the adequate synchronism (Asynchronous for real time cameras
		/// and Synchronous for non real time cameras), a set of three one channel images in the worker to the
		/// corresponding output image properties of the camera. An error (return value=false) will occur if any of
		/// the dstProp are not of the adequate QVImage<uChar,1> type.
		///
		/// @param dstCont Pointer to the destination worker.
		/// @param dstProp1 Name of the destination property for image Y.
		/// @param dstProp2 Name of the destination property for image U.
		/// @param dstProp3 Name of the destination property for image V.
		/// @return true if success, false otherwise.
		bool linkProperty(QVPropertyContainer *dstCont, QString dstProp1, QString dstProp2, QString dstProp3);

	protected:
		bool realTime, noLoop;

		/// @brief Pure virtual openCam method.
		///
		/// Reimplementations of this method should open the video source specified by the string urlName, 
		/// trying to open it with the suggested cols, rows and fps. The method must indicate a successful opening 
		/// by returning true (and updating the corresponding cols, rows and fps output values to the really obtained
		/// ones, which could coincide or not with the suggested input values), or false, if there was any problem
		/// opening the source.
		/// @param urlName QString determining the video source.
		/// @param cols Suggested number of columns (as input). On return contains real number of columns (as output).
		/// @param rows Suggested number of rows (as input). On return contains real number of rows (as output).
		/// @param fps Suggested fps (as input). On return contains real fps (as output).
		/// @return true if success, false if fail.
		virtual bool openCam(QString urlName, int &cols, int &rows, int &fps) = 0;

		/// @brief Pure virtual closeCam method.
		///
		/// Reimplementations of this method should close the video source. 
		virtual void closeCam() = 0;

		/// @brief Pure virtual grab method.
		///
		/// Reimplementations of this method should grab Y, U and V channel images in the corresponding variable
		/// parameters, with respective sizes cols x rows, cols/2 x rows/2 and cols/2 x rows/2 (with cols and rows as
		/// returned by the @ref openCam() method), returning true if grabbing was succesful, or false otherwise 
		/// (for example, if the camera was closed).
		/// @param imgY On return, contains the grabbed Y image.
		/// @param imgU On return, contains the grabbed U image.
		/// @param imgV On return, contains the grabbed V image.
		virtual bool grab(QVImage<uChar,1> &imgY, QVImage<uChar,1> &imgU, QVImage<uChar,1> &imgV) = 0;

		/// @brief Pure virtual method to get the length of the video.
		///
		/// Reimplementations of this method should inform of the length of the video in seconds. A length
		/// equal to 0.0 should be returned if not available (for example, for live cameras, remote streaming
		/// video, and the like)
		virtual double lengthOfVideo() = 0;

		/// @brief Pure virtual method to get the current time position in the video.
		///
		/// Reimplementations of this method should inform of the current position in the video in seconds.
		virtual double currentPos() = 0;

	private slots:
		/// @brief Pure virtual slot to set a new time position in the video.
		///
		/// Reimplementations of this method should set the current position in the video in seconds (if possible).
		virtual void setCurrentPos(double time_pos) = 0;

		/// @brief Reopens the camera, using possibly new input parameters.
		///
		/// When this slot is called, the running camera is immediately closed, and the current values of the
		/// input properties are immediately used to reopen the camera source.
		void reopen();

		/// @brief Resets the camera worker.
		///
		/// When this slot is called, the running camera is immediately closed, and the worker stopped.
		void resetCameraWorker();

		void grabFrame();

		void nothing() { /* Just to block connected workers if in real time (using Qt::BlockingQueuedConnection) */ };

	private:
		bool flush_pending_images;
		QString urlName;
		QVImage<uChar,1> imageY, imageU, imageV;

		void tryOpeningCam();
		void iterate();
		void connectOrDisconnectLinkedWorkers(bool connect_disconnect);

	signals:
		/// @brief Emitted each time a new video source is opened.
		void opened();

		/// @brief Emitted each time the video source is closed.
		void closed();

		/// @brief Emitted each time a new frame is grabbed.
		void grabbed();

	};

#endif
