/*
 *	Copyright (C) 2007, 2008, 2009. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvyuv4mpeg2cameraworker.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <QVYUV4MPEG2CameraWorker>
#include <qvipp.h>

#include <qvdefines.h>

bool QVYUV4MPEG2CameraWorker::openCam(QString fileName, int &cols, int &rows, int &fps)
	{
	total_frames = 0;
	total_time = 0.0;
	current_time = 0.0;

	// We initially save requested number or rows and cols:
	out_rows = rows;
	out_cols = cols;

	// RealTime flag:
	realTime = getPropertyValue<bool>("RealTime");

	// Try to open file, and read yuv4mpeg2 header.
	videoFile.setFileName(fileName);
	if (!videoFile.exists())
		{
		QString msg = QString("QVYUV4MPEG2CameraWorker::openCam: File '") + fileName + "' doesn't seem to exist." ;
		qWarning() << msg;
		setLastError(msg);
		return FALSE;
		}
	else if (!videoFile.open(QIODevice::ReadOnly))
		{
		QString msg = QString("QVYUV4MPEG2CameraWorker::openCam: Can't open file '") + fileName + "' in read mode." ;
		qWarning() << msg;
		setLastError(msg);
		return FALSE;
		}
	else if (!readYUV4MPEG2Header(videoFile, cols, rows, fps))
		{
		QString msg = QString("QVYUV4MPEG2CameraWorker::openCam: File '") + fileName + "' doesn't seem to be a valid yuv4mpeg2 video file.";
		qWarning() << msg;
		setLastError(msg);
		return FALSE;
		}
	else	// Everything correct.
		{
		// Size of the header:
		header_size = videoFile.pos();

		// cols, rows and fps were passed by variable (so they have now the real, input source video file values.
		inp_cols = cols;
		inp_rows = rows;

		// Output cols and rows:
		if(out_cols != 0 and out_rows != 0) 
			{
			// Even rounding of out_rows and out_cols:
			out_cols = out_cols + (out_cols%2);
			out_rows = out_rows + (out_rows%2);
			}
		else // cols==0 or rows==0 means default (input) size:
			{
			out_cols = inp_cols;
			out_rows = inp_rows;
			}

		// If requested number of cols or rows is different from original input video size, we will have to rescale:
		rescale = (inp_cols != out_cols) or (inp_rows != out_rows);

		// Finally, output cols and rows are "returned" to the caller:
		cols = out_cols; rows = out_rows;

		// We save FPS (useful for real time cameras):
		out_fps = fps;

		// We compute the total number of frames of the video (file size divided by size of each frame). Header
		// size is discounted, and then the 6 constant comes from the "Frame\n" string at the beginning of each frame:
		total_frames = (videoFile.size()-header_size) / (inp_cols*inp_rows + inp_cols*inp_rows/2 + 6);

		// Finally, we compute the time length in seconds:
		total_time = static_cast<double>(total_frames) / out_fps;

		return TRUE;
		}
	}

void QVYUV4MPEG2CameraWorker::closeCam()
	{
	videoFile.close();
	}

bool QVYUV4MPEG2CameraWorker::grab(QVImage<uChar,1> &imgY, QVImage<uChar,1> &imgU, QVImage<uChar,1> &imgV)
	{
	QVImage<uChar,1> inp_imgY(inp_cols,inp_rows), inp_imgU(inp_cols/2,inp_rows/2), inp_imgV(inp_cols/2,inp_rows/2);
	bool ret_value = FALSE;

	// Simulated read delay for real time cameras:
	static QTime last_time;
	if(realTime)
		{
			int msecs_to_wait = (int)(1000/(double)out_fps) - last_time.elapsed();
			if((msecs_to_wait > 0) and (msecs_to_wait < (int)(1000/(double)out_fps)))
				msleep(msecs_to_wait);
		}

	if (readYUV4MPEG2Frame(videoFile, inp_imgY, inp_imgU, inp_imgV))
		{
		ret_value = TRUE;
		}
	else if (videoFile.atEnd())
		{
		if (noLoop)
			return FALSE;
		else
			{
			qDebug() << "QVYUV4MPEG2CameraWorker::grabFrame(): Reseting the video at frame " << getPropertyValue<int>("Frames");
			videoFile.reset();
			readYUV4MPEG2Frame(videoFile, inp_imgY, inp_imgU, inp_imgV);
			ret_value = TRUE;
			}
		}
	else	{
		qFatal("QVYUV4MPEG2CameraWorker::grab(): readYUV4MPEG2Frame() returned FALSE, and videoFile is not at end... aborting\n");
		}
	
	// Possible rescaling:
	if(rescale)
		{
		Resize(inp_imgY,imgY);
		Resize(inp_imgU,imgU);
		Resize(inp_imgV,imgV);
		}
	else
		{
		imgY = inp_imgY;
		imgU = inp_imgU;
		imgV = inp_imgV;
		}

	if(realTime)
		last_time.start();

	return ret_value;
	}

double QVYUV4MPEG2CameraWorker::lengthOfVideo()
	{
	return total_time;
	}

double QVYUV4MPEG2CameraWorker::currentPos()
	{
	return total_time*videoFile.pos()/videoFile.size();
	}

void QVYUV4MPEG2CameraWorker::setCurrentPos(double time_pos)
	{
	int frame = (int)((time_pos*total_frames)/total_time);
	videoFile.seek(header_size+frame*(inp_cols*inp_rows + inp_cols*inp_rows/2 + 6));
	}
